#define TIMER_CLOCK_FREQ 16000.0 //16kHz for /1024 prescale from 16MHz

int Timer2Cnt;
long latency;
long timerLoadValue;

int CurrPowerValue;
int RMC3ShimPin = 3;

void setup() {
  pinMode(RMC3ShimPin, OUTPUT);

  Serial.begin(9600);
  Serial.println("Timer NANO");
  CurrPowerValue = 0;
  // Тут задается период ШИМа в секундах умноженный на 100 для точности регулировки мощности РМЦ в 1%
  // В данном примере это 200/100 = 2Гц или 0,5мс
  timerLoadValue = SetupTimer2(200);
}

unsigned char SetupTimer2(float timeoutFrequency) {
  unsigned char result; //Начальное значение таймера.
  //Подсчет начального значения таймера
  result = (int)((257.0 - (TIMER_CLOCK_FREQ / timeoutFrequency)) + 0.5);
  //257 на самом деле должно быть 256, но я получил лучшие результаты с 257.
  //Установки Таймер2: Делитель частоты /1024, режим 0
  //Частота = 16MHz/1024 = 16khz
  TCCR2A = 0;
  TCCR2B = 1 << CS22 | 1 << CS21 | 1 << CS20;
  //Подключение прерывания по переполнению Timer2
  TIMSK2 = 1 << TOIE2;
  //загружает таймер для первого цикла
  TCNT2 = result;
  return (result);
}

//Timer2 указатель вектора прерывания по переполнению
ISR(TIMER2_OVF_vect) {
  if (Timer2Cnt <= CurrPowerValue)
  {
    //Переключение IO-вывода во ВКЛюченное состояние.
    digitalWrite(RMC3ShimPin, HIGH);
  }
  else
  {
    //Переключение IO-вывода в ВЫКЛюченное состояние.
    digitalWrite(RMC3ShimPin, LOW);
  }

  if (Timer2Cnt < 100)
  {
    Timer2Cnt++;
  }
  else
  {
    Timer2Cnt = 0;
  }
  //Захват текущего значения таймера. Это величина ошибки
  //из-за задержки обработки прерывания и работы этой функции
  latency = TCNT2;
  //Перезагрузка таймера и коррекция по задержке
  TCNT2 = latency + timerLoadValue;
}

void loop() {
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(10000);

  CurrPowerValue = 20;
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(30000);

  CurrPowerValue = 50;
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(30000);

  CurrPowerValue = 70;
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(30000);

  CurrPowerValue = 93;
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(30000);

  CurrPowerValue = 100;
  Serial.print("Power, % - ");
  Serial.println(CurrPowerValue);
  delay(30000);
}
